//---------------------------------------------------------
//RG Basic Tools Collection
//Copyright © 2003 - 2005 Ronald D. Grindle
//---------------------------------------------------------

/*
 * Created on Apr 23, 2005
 */
package com.qanavigation.Basics;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.lang.Thread;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Date;

/**
* A collection of little helpers
*/
public class BTools {
	
	//------------------------------------
	/**
	 * @return The date from the ISO format YYYY-MM-DDTHH:mm
	 */
	public static String getDateFromISODateTime( String p_DateTime ) {

		String ISODate = "";

		int pos;

		pos = p_DateTime.indexOf( 'T' );

		if( pos > 0 ) {

			ISODate = p_DateTime.substring( 0, pos );
		}

		return ISODate;
	}

	//----------------------------------------
	/**
	* @return a date in ISO format YYYY-MM-DD
	*/
	public static String getISODate( Calendar p_Date ) {

		String ISOtime;

		ISOtime = String.valueOf( p_Date.get( Calendar.YEAR ) ) + "-" + IntStr2( p_Date.get( Calendar.MONTH ) + 1 ) + "-" + IntStr2( p_Date.get( Calendar.DAY_OF_MONTH ) );

		return ISOtime;
	}
	
	//-------------------------------------------------------------------------
	/// <summary>
	/// Returns a date/time in ISO format YYYY-MM-DDTHH:mm
	/// </summary>
	/// <returns>string</returns>
	public static String getISODateTime( Calendar p_Date ) 
	{
		String ISOdTime;

		ISOdTime = String.valueOf( p_Date.get( Calendar.YEAR ) ) + "-" + IntStr2( p_Date.get( Calendar.MONTH ) + 1 ) + "-" + IntStr2( p_Date.get( Calendar.DAY_OF_MONTH ) );
		ISOdTime += "T" + IntStr2( p_Date.get( Calendar.HOUR_OF_DAY ) ) + ":" + IntStr2( p_Date.get( Calendar.MINUTE ) );

		return ISOdTime;
	}
	
	//-------------------------------------------------------------------------
	/// <summary>
	/// Returns a date/time in ISO format YYYY-MM-DDTHH:mm:ss
	/// </summary>
	/// <returns>string</returns>
	public static String getISODateTimeSec( Calendar p_Date ) 
	{
		String ISOdTime;

		ISOdTime = String.valueOf( p_Date.get( Calendar.YEAR ) ) + "-" + IntStr2( p_Date.get( Calendar.MONTH ) + 1 ) + "-" + IntStr2( p_Date.get( Calendar.DAY_OF_MONTH ) );
		ISOdTime += "T" + IntStr2( p_Date.get( Calendar.HOUR_OF_DAY ) ) + ":" + IntStr2( p_Date.get( Calendar.MINUTE ) ) + ":" + IntStr2( p_Date.get( Calendar.SECOND ) );

		return ISOdTime;
	}	
	
	//-------------------------------------------------------------------------
	/// <summary>
	/// Returns a date/time in ISO format YYYY-MM-DDTHH:mm
	/// </summary>
	/// <returns>string</returns>
	public static String getISODateTime( long p_Date ) 
	{
		String ISOdTime;
		
		GregorianCalendar gc = new GregorianCalendar();
		
		gc.setTime( new Date( p_Date ) );
		
		ISOdTime = getISODateTime( gc );

		return ISOdTime;
	}

	//-----------------------------------------
	/**
	* @return the current date in ISO format YYYY-MM-DD
	*/
	public static String getNowISODate( ) {

		String ISOtime;
		
		Calendar rightNow = Calendar.getInstance();

		ISOtime = getISODate( rightNow );

		return ISOtime;
	}
	
	//-------------------------------
	/**
	 * @return the current date/time in ISO format YYYY-MM-DDTHH:mm
	 */
	public static String getNowISODateTime( ) {

		String ISOdTime;

		Calendar rightNow = Calendar.getInstance();

		ISOdTime = getISODateTime( rightNow );

		return ISOdTime;
	}
	
	//-------------------------------
	/**
	 * @return the current date/time in ISO format YYYY-MM-DDTHH:mm
	 */
	public static String getNowISODateTimeSec( ) {

		String ISOdTime;

		Calendar rightNow = Calendar.getInstance();

		ISOdTime = getISODateTimeSec( rightNow );

		return ISOdTime;
	}

	//-----------------------------------------
	/**
	* @return the current time in short ISO format HH:mm
	*/
	public static String getNowISOTime( ) {

		String ISOTime;

		Calendar rightNow = Calendar.getInstance();

		ISOTime = IntStr2( rightNow.get(Calendar.HOUR_OF_DAY ) ) + ":" + IntStr2( rightNow.get( Calendar.MINUTE ) );
		
		return ISOTime;
	}

	//-----------------------------------------
	/**
	* @return the current time in short ISO format HH:mm:ss
	*/
	public static String getNowISOTimeSec( ) {

		String ISOTime;

		Calendar rightNow = Calendar.getInstance();

		ISOTime = IntStr2( rightNow.get(Calendar.HOUR_OF_DAY ) ) + ":" + IntStr2( rightNow.get( Calendar.MINUTE ) ) + ":" + IntStr2( rightNow.get( Calendar.SECOND ) );
		
		return ISOTime;
	}
	
	//	------------------------------
	/**
	 * @return the time from ISO format YYYY-MM-DDTHH:mm
	 */
	public static String getTimeFromISODateTime( String p_DateTime ) {

		String ISOTime = "";

		int pos;

		pos = p_DateTime.indexOf( 'T' );
		
		if( pos > -1 ) {

			ISOTime = p_DateTime.substring( pos + 1 );
		}

		return ISOTime;
	}
	
	/**
	 * Expects a string in the format YYYY-MM-DDTHH:mm
	 * @param p_ISODateTime
	 * @return The time in milliseconds, or -1 in case of an error.
	 */
	public static long getTimeInMillis( String p_ISODateTime ) {
		
		int year, month, date, hourOfDay, minute;
		
		long mil;
		
		String sub;
		
		try {
			
			Calendar cal = Calendar.getInstance();
			
			sub = p_ISODateTime.substring( 0, 4);
			year = Integer.valueOf( sub );
			
			sub = p_ISODateTime.substring( 5, 7);
			month = Integer.valueOf( sub );
			month--;
			
			sub = p_ISODateTime.substring( 8, 10);
			date = Integer.valueOf( sub );
			
			sub = p_ISODateTime.substring( 11, 13);
			hourOfDay = Integer.valueOf( sub );
			
			sub = p_ISODateTime.substring( 14, 16);
			minute = Integer.valueOf( sub );		
			
			cal.clear();
			cal.set( year, month, date, hourOfDay, minute );
			
			mil = cal.getTimeInMillis();
		}
		catch( Exception excp ) {
			
			mil = -1;
		}
		
		return mil;
	}
	
	/**
	 * 
	 * @param p_Int
	 * @return
	 */
	public static String IntStr2( int p_Int ) {
	
		String padInt;
		String intStr;
		
		padInt = "";
		
		intStr = String.valueOf( p_Int ); 
			
		if( intStr.length() < 2 ) {
			
			padInt = "0";
		}
		
		padInt += intStr;
		
		return padInt;
	}
	
	/**
	 * Fills a string with leading zeros.
	 * @param p_Num The string
	 * @param p_Len resulting length.
	 * @return
	 */
	public static String padLZeros( String p_Num, int p_Len ) {
		
		int i, x;
		
		String padStr = "";

		x = p_Len - p_Num.length();
		
		for( i = 0; i < x; i++ ) {
			
			padStr += "0";
		}
		
		padStr += p_Num;
		
		return padStr;
	}	
	
	/**
	 * Extract a stream into a RsltString.
	 * Stuffs everything in one line, regardless of any line breaks.
	 * @param p_IStrm
	 * @return
	 */
	public static RsltString streamIntoString( InputStream p_IStrm ) {

		RsltString rstr = new RsltString();
		
		String line;
		String str = "";
		
		try {
			
			BufferedReader br = new BufferedReader(new InputStreamReader( p_IStrm ));
			
			while( !(br.ready()) ) {
				
				Thread.sleep(100);
			}
			
			line = br.readLine();
					
	    while( null != line ) {
	      
				str += line;
	    	
				/*while( !(br.ready()) ) {
					
					Thread.sleep(100);
				}*/
				
				line = br.readLine();			
			}
			
			br.close();
			
			rstr.setString( str );
		}
		catch( Exception excp ) {
				
			rstr.setRsltByExcp( excp );
		}		
/* bad version
		byte[] barr;
		
		int x;
		RsltString rstr = new RsltString( true );
		String line;
		String str = "";
		
		try {
		
			x = p_IStrm.read();
		
			while( x > 0 ) {
				
				barr = new byte[x];
	
				p_IStrm.read( barr, 0, x );
					
				line = new String( barr );
				
				str += line;
				
				x = p_IStrm.read();
			}
		
			rstr.setString( str );
		}
		catch( Exception excp ) {
			
			rstr.setRsltByExcp( excp );
		}
*/		
		return rstr;
	}		
}
