//---------------------------------------------------------
// RG Basic Tools Collection
// Copyright © 2003,2004 Ronald D. Grindle
//---------------------------------------------------------
package com.qanavigation.XMLBasics;

import org.w3c.dom.*;

import com.qanavigation.Basics.Rslt;
import com.qanavigation.Basics.RsltStatVal;
import com.qanavigation.Basics.RsltString;
import com.qanavigation.Basics.SFormatter;


/**
 * XML node as a result class + some helpers.
 * Creates its own XMLDoc!
 * RsltXMLNode isA Rslt.
 */
public class RsltXMLNode extends XMLRslt {


	public Document m_xDoc;

	//public Element m_xElem;

	public Node m_xNode;

	/**
	 * Creates an undefined node!
	 */
	public RsltXMLNode(  ) {

		setInError( "Construction w/o node definition (RsltXMLNode()). Invalid until defined!" ); 
	}

	/**
	 * 
	 * @param p_XElem
	 */
	public RsltXMLNode( Element p_XElem ) {

		m_xDoc = p_XElem.getOwnerDocument();

		//m_xElem = p_XElem;
		
		m_xNode = p_XElem;

		setOK();
	}
	
	/**
	 * 
	 * @param p_XNode
	 */
	public RsltXMLNode( Node p_XNode ) {

		m_xDoc = p_XNode.getOwnerDocument();

		//m_xElem = null;
		
		m_xNode = p_XNode;

		setOK();
	}

	/**
	 * 
	 * @param p_XMLNode
	 */
	public RsltXMLNode( RsltXMLNode p_XMLNode ) {

		m_xDoc = p_XMLNode.m_xDoc;

		//m_xElem = p_XMLNode.m_xElem;

		m_xNode = p_XMLNode.m_xNode;

		setRsltStat( p_XMLNode );
	}

	/**
	 * The node is automatically appended to the XMLDoc!
	 * @param p_NodeName
	 */
	public RsltXMLNode( String p_NodeName ) {

		init( p_NodeName );
	}
	
	/**
	 * 
	 * @param p_Name
	 * @param p_Val
	 */
	public void addAttrAsHTML( String p_Name, String p_Val ) {
		
		NamedNodeMap nMap;
		
		String val;
		
		try {

			Attr nAttr = m_xDoc.createAttribute( p_Name );
			
			val = SFormatter.encodeHTML( p_Val );
			
			nAttr.setNodeValue( val );
				
			nMap = m_xNode.getAttributes();
			
			nMap.setNamedItem( nAttr );
		}
		catch( Exception excp ) {

			setRsltByExcp( excp );
			addRsltAddInfo( "'com.qanavigation.XMLBasics.RsltXMLNode.addAttribute(..)' failed." );
		}
	}

	/**
	 * 
	 * @param p_Name
	 * @param p_Val
	 */
	public void addAttribute( String p_Name, String p_Val ) {
		
		NamedNodeMap nMap;
		
		try {

			Attr nAttr = m_xDoc.createAttribute( p_Name );
			
			nAttr.setNodeValue( p_Val );
				
			nMap = m_xNode.getAttributes();
			
			nMap.setNamedItem( nAttr );
		}
		catch( Exception excp ) {

			setRsltByExcp( excp );
			addRsltAddInfo( "'com.qanavigation.XMLBasics.RsltXMLNode.addAttribute(..)' failed." );
		}
	}
	
	/**
	 * 
	 * @param p_Comment
	 */
	public void addComment( String p_Comment ) {

		try {

			Comment xCom;
			
			//xDoc = m_xNode.getOwnerDocument();

			xCom = m_xDoc.createComment( p_Comment );

			Element root = m_xDoc.getDocumentElement();
			m_xDoc.insertBefore( xCom, root );
		}
		catch( Exception excp ) {

			setRsltByExcp( excp );
			addRsltAddInfo( "'" + this.getClass().getName() + ".addComment()' failed." );
		}
	}

	// --------------------------------------------------------------------------
	/* not supported in DOM2
	 * can only be specified at construction time of document! 
	 *
	public void addPubDocTypeInfo( String p_NodeName, String p_DTDname ) {

		try {

			DocumentType docType;

			docType = m_xDoc.CreateDocumentType( p_NodeName, p_DTDname, null, null  );

			XmlElement root = m_xDoc.DocumentElement;
			m_xDoc.InsertBefore(docType, root);
		}
		catch( Exception excp ) {

			setResByExcp( excp );
			addResAddInfo( "'com.qanavigation.XMLBasics.RsltXMLNode.addPubDocTypeInfo(..)' failed." );
		}
	}
	*/

	// --------------------------------------------------------------------------
	/* not supported in DOM2.
	 * can only be specified at construction time of document! 
	 *
	public void addSysDocTypeInfo( String p_NodeName, String p_DTDname ) {

		try {

			XmlDocumentType docType;

			docType = m_xDoc.CreateDocumentType( p_NodeName, null, p_DTDname, null  );

			XmlElement root = m_xDoc.DocumentElement;
			m_xDoc.InsertBefore(docType, root);
		}
		catch( Exception excp ) {

			setResByExcp( excp );
			addResAddInfo( "'com.qanavigation.XMLBasics.RsltXMLNode.addSysDocTypeInfo(..)' failed." );
		}
	}
	*/


	/**
	 * Adds a UID, if a UID is provided.
	 * @param p_UID
	 */
	public void addUID( String p_UID ) {

		try {

			if( p_UID.length() > 0 ) {
				
				//m_xNode.setAttribute( "UID", p_UID );
				
				addAttribute( "UID", p_UID );			
			}
		}
		catch( Exception excp ) {

			setRsltByExcp( excp );
			addRsltAddInfo( "'com.qanavigation.XMLBasics.RsltXMLNode.addUID(..)' failed." );
		}
	}

	/**
	 * 
	 * @param p_NodeName
	 * @return
	 */
	public RsltXMLNode appNode( String p_NodeName ) {
		
		RsltXMLNode newNode = new RsltXMLNode();

		try {
			
			newNode.m_xDoc = m_xDoc;

			newNode.m_xNode = m_xDoc.createElement( p_NodeName );

			//newNode.m_xNode = m_xNode.appendChild( newNode.m_xNode );
			m_xNode.appendChild( newNode.m_xNode );

			newNode.setOK();

			setOK();
		}
		catch( Exception excp ) {

			setRsltByExcp( excp );
			addRsltAddInfo( "'com.qanavigation.XMLBasics.RsltXMLNode.appNode(1)' failed." );

			newNode.setRsltStat( this );
		}

		return newNode;
	}
	
	/**
	 * 
	 * @param p_Node
	 * @return
	 */
	public Rslt appNode( RsltXMLNode p_Node ) {
		
		Rslt res = new Rslt( true );
		
		Node node;

		try {		
			
			node = m_xDoc.adoptNode( p_Node.m_xNode );
			
			m_xNode.appendChild( node );
		}
		catch( Exception excp ) {
			
			setRsltByExcp( excp );
			addRsltAddInfo( "'com.qanavigation.XMLBasics.RsltXMLNode.appNode(2)' failed." );
			
			res.setRsltStat( this );
		}
		
		return res;
	}

	/**
	 * Appends a value (text) to this node.
	 */
	public RsltXMLNode appTxt( String p_NodeVal ) {
		
		RsltXMLNode newNode = new RsltXMLNode();

		try {

			Text txtNode;

			txtNode = m_xDoc.createTextNode( p_NodeVal );

			//newNode.m_xNode = m_xNode.appendChild( txtNode );
			m_xNode.appendChild( txtNode );

			newNode.setOK();

			setOK();
		}
		catch( Exception excp ) {

			setRsltByExcp( excp );
			addRsltAddInfo( "'com.qanavigation.XMLBasics.RsltXMLNode.appTxt(..)' failed." );

			newNode.setRsltStat( this );
		}

		return newNode;
	}

	/**
	 * Creates a new text node and appends it to the current.
	 */
	public RsltXMLNode appTxtNode( String p_NodeName, String p_NodeVal ) {

		RsltXMLNode newNode = appNode( p_NodeName );

		newNode.appTxt( p_NodeVal );

		newNode.setOK();

		setOK();

		return newNode;
	}

	// --------------------------------------------------------------------------
/*
	public void createXmlDecl( String p_XMLver, String p_Encoding, String p_Standalone ) {

		try {

			Declaration xmlDecl;
			
			m_xDoc.

			xmlDecl = m_xDoc.createXmlDeclaration( p_XMLver, "", "" );

			XmlElement root = m_xDoc.DocumentElement;
			m_xDoc.InsertBefore(xmlDecl, root);

			setOK();

		//m_xDoc.AppendChild( xmlDecl );
		}
		catch( Exception excp ) {

			setResByExcp( excp );
			addResAddInfo( "'com.qanavigation.XMLBasics.RsltXMLNode.createXmlDecl(..)' failed." );
		}
	}
	*/


	// --------------------------------------------------------------------------
	/* soup up with xpath abilities asap */
	public RsltString getAttrVal( String m_AttrName ) {
		
		RsltString resVal = new RsltString();
		
		NamedNodeMap nMap;
		Node node;
		
		try {
			//resVal.setString( m_xNode.getAttribute( m_AttrName ) );
			
			nMap = m_xNode.getAttributes();
			
			node = nMap.getNamedItem( m_AttrName );
			
			if( null != node ) {
				
				resVal.setString( node.getNodeValue() );
			}
		}
		catch( Exception excp ) {
			
			resVal.setRsltByExcp( excp );
			resVal.addRsltAddInfo( "'com.qanavigation.XMLBasics.RsltXMLNode.getAttrVal(..)' failed." );
		}
		
		return resVal;
	}


	/**
	 * Only succesful if the node is unique
	 */
/*	
	public RsltXMLNode getSubNode( String p_XPath ) {

		RsltXMLNode resNode;

		try {
		
			NodeList nodeLst;
			Node theNode;

			nodeLst = m_xNode.selectNodes( p_XPath );

			if( 1 != nodeLst.getLength() ) {
				
				resNode = new RsltXMLNode();
				resNode.setInError( "Node '" + p_XPath + "' not found or not unique." );
			}
			else {

				theNode = nodeLst.item( 0 );
				resNode = new RsltXMLNode( theNode );
			}
		}
		catch( Exception excp ) {

			setResByExcp( excp );
			addResAddInfo( "'com.qanavigation.XMLBasics.RsltXMLNode.getSubNode(..)' failed." );

			resNode = new RsltXMLNode();
			resNode.setResStat( this );
		}

		return resNode;
	}
*/
	
	public RsltXMLNodeLst getSubNodes( String p_XPath ) {
	
		RsltXMLNodeLst theLst;
		RsltXMLNodeLst aLst;
		RsltXMLNodeLst bLst;
		
		boolean notF;
		int i, s, x;
		
		Node aNode;
		
		RsltXMLNode	rNode;
		
		String name;
		
		aNode = null;
		
		theLst = new RsltXMLNodeLst();
		
		//rNode = new RsltXMLNode();
		
		try {
	
			s = p_XPath.indexOf( "/" );
			
			if( s == -1 ) {
				
				name = p_XPath;
			}
			else {
				
				name = p_XPath.substring( 0, s );
			}
			
			NodeList nLst = m_xNode.getChildNodes();
			
			x = nLst.getLength();
			
			notF = true;
			
			for( i = 0; i < x; ++i ) {
				
				aNode = nLst.item( i );
				
				if( aNode.getNodeName().compareTo(name) == 0) {
					
					rNode = new RsltXMLNode( aNode );
					
					theLst.add( rNode );
					notF = false;
				}
			}
		
			if( notF ) {
				
				theLst.setInError( "A Node '" + name + "' could not be found." );
				theLst.setRsltStat( RsltStatVal.isEmpty );
			}
			else {
						
				theLst.setOK();
						
				if( s > -1 ) {
					
					name = p_XPath.substring( s + 1 );
					
					x = theLst.size();
					
					bLst = new RsltXMLNodeLst();
					
if( 0 == x ) {
	
	bLst.addRsltAddInfo( "The list is empty" );
}
					
					for( i = 0; i < x; ++i ) {
						
						rNode = theLst.getNode( i ); 
						
						aLst = rNode.getSubNodes( name  );
						
						bLst.addRange( aLst );
						
						bLst.checkRslt( aLst );
					}
					
					theLst = bLst;
				}
			}		
		}
		catch( Exception excp ) {
	
			theLst.setRsltByExcp( excp );
		}			
		
		return theLst;
	}
	
	/* Replaced by getSubNodes
	 *  replace by xpath abilities in Java 1.5  asap
	 *					
	public RsltXMLNode getSubNode( String p_XPath ) {
		
		boolean notF;
		int i, s, x;
		
		Node aNode;
		
		RsltXMLNode	rNode;	
		
		String val;
		
		String name;
		
		aNode = null;
		
		rNode = new RsltXMLNode();
		
		try {
	
			s = p_XPath.indexOf( "/" );
			
			if( s == -1 ) {
				
				name = p_XPath;
			}
			else {
				
				name = p_XPath.substring( 0, s );
			}
			
			NodeList nLst = m_xNode.getChildNodes();
			
			x = nLst.getLength();
			
			notF = true;
			
			for( i = 0; (i < x) & notF; ++i ) {
				
				aNode = nLst.item( i );
				
				if( aNode.getNodeName().compareTo(name) == 0) {
				
					notF = false;
				}
			}
		
			if( notF ) {
				
				//rVal.setInError( "Path '" + p_ValPth + "' not found or is not unique." );
				aNode = null;
				
				rNode.setRsltStat( RsltStatVal.isEmpty );
			}
			else {
				
				rNode.init( aNode );
				
				if( s > -1 ) {
					
					name = p_XPath.substring( s + 1 );					
					
					rNode = rNode.getSubNode( name  );
				}

			}		
		}
		catch( Exception excp ) {
	
			rNode.setRsltByExcp( excp );
		}			

		return rNode;
	}*/
	
	/** Retrieves a single value from the document.
	 * @param p_XPath The node path, separated by '/'
	 */
	public RsltString getSubNodeVal( String p_XPath ) {

		RsltString xVal;
		RsltXMLNode xNode;

		xVal = new RsltString();

		RsltXMLNodeLst xNodes = getSubNodes( p_XPath );

		if( xNodes.notOK() ) {

			xVal.setRsltStat( xNodes );
		}
		else {

			xNode = xNodes.getNode( 0 );
			
			xVal.setString( xNode.getText().getString() );
		}
		
		return xVal;
	}

	/**
	 *  Text value of the node.
	 */
	public RsltString getText() { 

		Node cNode;
		RsltString txt;

		txt = new RsltString();
			
		NodeList cNodes = m_xNode.getChildNodes();

		if( 0 < cNodes.getLength() ) {

			cNode = cNodes.item( 0 );

			txt.setString( cNode.getNodeValue() );
		}

		return txt;
	}


	// --------------------------------------------------------------------------
	/*
	public static RsltXMLNode RsltXMLNode( Node p_Node ) {

		return new RsltXMLNode( p_Node );
	}
	*/

	/**
	 * Gets the UID, if there's a UID.
	 */
	public String getUID() {

		RsltString UID;

		UID = getAttrVal( "UID" );

		return UID.getString();
	}
	
	// --------------------------------------------------------------------------
	/*protected void init( Node p_Node ) {

		m_xDoc = p_Node.getOwnerDocument();

		m_xElem = p_Node.
		m_xNode = p_Node;

		setOK();
	}
	*/
	
	// --------------------------------------------------------------------------
	protected void init( RsltXMLNode p_rNode ) {

		m_xDoc = p_rNode.m_xDoc;

		//m_xElem = p_rNode.m_xElem;
		m_xNode = p_rNode.m_xNode;
		
		setRsltStat( p_rNode );
		
	}

	// --------------------------------------------------------------------------
	protected void init( String p_NodeName ) {

		Element elem;
		
		RsltXMLDoc xDoc = new RsltXMLDoc( "", p_NodeName );
		
		m_xDoc = xDoc.m_xDoc;

		//m_xElem = m_xDoc.getDocumentElement();
		//m_xNode = (Node)m_xElem;
		
		elem = m_xDoc.getDocumentElement();
		m_xNode = elem;

		setOK();
	}
	
	
	/**
	 *  Text value of the node.
	 */
	public Rslt setText( String p_Txt ) { 

		Node cNode;
		Rslt res = new Rslt( true );
		
		try {
			
			NodeList cNodes = m_xNode.getChildNodes();
	
			if( 0 < cNodes.getLength() ) {
	
				cNode = cNodes.item( 0 );
	
				cNode.setNodeValue( p_Txt );
			}
			else {
				
				appTxt( p_Txt );
			}
		}		
		catch( Exception excp ) {
			
			res.setRsltByExcp( excp );
		}				

		return res;
	}
}
